; ----------------------------------------------------------------------------
;
; Name, Version & History
; TargetPoint
; Author	TG Tan		www.angelfire.com\space2\tgtan
; V1.02		28/1/07		1st version (beta)
; V1.10		10/2/07		Included ability to select the path and filename of a target list
; V1.20		17/4/09		Added wait time when next target is on the other side of the Milky Way, added wait time countdown in status box 
; 
; Author:         TG Tan
;
; Script Function:
;  Controls both Artemis Capture and IRIS to point the telescope at successive targets on a list, and to take a defined set of images
;
; Assumptions:
; 1.  IRIS is open and the telescope control panel is open
; 2.  Artemis Capture is open, the timer window is open
; 3.  Input file format is Target name, RA, Dec in the form "AAAAA.... HH MM.mm DD MM.mm"
; ----------------------------------------------------------------------------
; Note: there is a bug in here that prevents the script ending as intended, with a message.  However functioning of the script is not affected
; - so if it ain't broke, don't fix it!

;**************************************************************
;  some initial preparations...
;***************************************************************

;#include <Date.au3>
#include <file.au3>


HotKeySet("^!x", "My_Final_Exit") ; exit the script if Ctrl Alt x is pressed
HotKeySet("^!p", "Pause") ; on Ctrl Alt p, pause the script

Global $no_exp
Global $paused
Global $file
Global $params_file
Global $targets_total = 0
Global $targets_done
Global $telescope_settle_time
Global $path
Global $targetlist_filename
Global $exp_time
Global $ccd_bin
Global $script_has_started
Global $prev_RA_in_mins
Global $current_RA_in_mins
Global $Milky_Way_wait_time		; if difference in RA from last target is more than this, wait this time before starting CCD exposure - in minutes

;******************************************************************

$Milky_Way_wait_time = 180	; adjust to suit typical RA jump across the Milky Way


;******************************************************************
$targets_done = 0		; no of targets that have been finished

$Art_exp_time_ctrl = 1000
$Art_binx_ctrl = 1002
$Art_biny_ctrl = 1003
$Art_autosave_ctrl = 1030

$Art_filename_ctrl = 1031
$Art_fileindex_ctrl = 1035		; Control ID for the Artemis exposure settings window

$iris_ra_ctrl = 1126
$iris_dec_ctrl = 1127
$iris_goto_ctrl = 1119; Control ID for the telescope control panel

$art_winposx = 328
$art_winposy = 82; where to place the Artemis Capture window

$Art_exp_x = 602
$Art_exp_y = 152; position of the single exposure button

$script_has_started = 0; set to 1 when the script has run through at least once
; used to signal when it's possible to re-start exposures after the Pause button has been pressed

$paused = 0 ; set to 1 when Ctrl Alt P has been pressed to signal that the script is paused


;*************************************************************************
; initialise the start values of the other variables, to be used on initial run, or if the params file doesn't exist
;*************************************************************************
If FileExists(@ScriptDir & "\" & "param_targetpoint.txt") Then
	$params_file = FileOpen(@ScriptDir & "\" & "param_targetpoint.txt", 0)
	
	$no_exp = FileReadLine($params_file)						; the number of exposures per target
	$exp_time = FileReadLine($params_file)						; exposure time per frame in s
	$ccd_bin = FileReadLine($params_file)						; the image binning - always n x n
	$path = FileReadLine($params_file)							; where to save image files to
	$targetlist_filename = FileReadLine($params_file)
	$telescope_settle_time = FileReadLine($params_file)		; allows time for scope to slew to next target and settle

	FileClose($params_file)
	
Else; on an initial run, use these defaults
	$no_exp = 10
	$exp_time = 20
	$ccd_bin = 2
	$path = "D:\Astro\"
	$targetlist_filename = ""
	$telescope_settle_time = 40		; this is in s
	
EndIf

;*****************************************************************************
; This block contains code for the script graphical interface
;******************************************************************************

#include <GuiConstants.au3>
opt("GuiOnEventMode", 1) ; change to On Event mode


$main_window = GUICreate("TargetPoint, V1.2 by TG Tan", 390, 385, 0, 0)


; get input for the number of exposures per target

GUICtrlCreateLabel("No. of exposures per target:", 10, 21)
$no_exp_box = GUICtrlCreateInput($no_exp, 170, 16, 50, 20, $ES_NUMBER)

; get input for the exposure time per frame

GUICtrlCreateLabel("Exposure time per frame (s):", 10, 49)
$exp_time_box = GUICtrlCreateInput($exp_time, 170, 45, 50, 20, $ES_NUMBER)


; get input for image binning

GUICtrlCreateLabel("CCD pixel binning ( n x n ):", 10, 77)
$ccd_bin_box = GUICtrlCreateInput($ccd_bin, 170, 72, 50, 20, $ES_NUMBER)

; get input for the path to which images should be saved

GUICtrlCreateLabel("Image save path:", 10, 105)
$path_box = GUICtrlCreateInput($path, 170, 100, 140, 20, $ES_AUTOHSCROLL)

; make a Browse button for the path

$browse_button = GUICtrlCreateButton("Browse", 330, 92, 50, 30)
GUICtrlSetOnEvent($browse_button, "Browse_path") ; when button is pressed, locate the desired path

; get input for the name of the target list file

GUICtrlCreateLabel("Target list file:", 10, 133)
$targetlist_filename_box = GUICtrlCreateInput($targetlist_filename, 170, 128, 140, 20, $ES_AUTOHSCROLL)

; make a File Select button

$select_button = GUICtrlCreateButton("Select", 330, 126, 50, 30)
GUICtrlSetOnEvent($select_button, "Select_file") ; when button is pressed, locate the file


; get input for telescope settle time allowed

GUICtrlCreateLabel("Scope settle time after slew (s):", 10, 163)
$telescope_settle_time_box = GUICtrlCreateInput($telescope_settle_time, 170, 158, 50, 20, $ES_NUMBER)

;*********************************************************************************
; the status display part of the GUI
;*********************************************************************************

; make a display for the current target

$current_target_label = GUICtrlCreateLabel("Current Target:", 40, 255, 190, 60)
GUICtrlSetFont($current_target_label, 11, 600)
$current_target_box = GUICtrlCreateInput("Not started", 180, 250, 190, 25, $ES_READONLY)
GUICtrlSetFont($current_target_box, 13, 500)

; make a display that keeps track of the no. of targets done

$targets_done_label = GUICtrlCreateLabel("Targets done:", 40, 292, 160, 50)
GUICtrlSetFont($targets_done_label, 12, 600)
$targets_done_box = GUICtrlCreateInput($targets_done, 180, 285, 70, 32, $ES_READONLY)
GUICtrlSetFont($targets_done_box, 20, 800)

$targets_total_label = GUICtrlCreateLabel("of", 260, 292, 160, 50)
GUICtrlSetFont($targets_total_label, 12, 600)
$targets_total_number = GUICtrlCreateLabel($targets_total, 300, 287, 160, 50)
GUICtrlSetFont($targets_total_number, 20, 800)

; make a display for status

$status_label = GUICtrlCreateLabel("Status:", 10, 327)
GUICtrlSetFont($status_label, 9, 400)
$status_displ_box = GUICtrlCreateInput("Press RUN to start...", 50, 324, 320, 22, $ES_READONLY)
GUICtrlSetFont($status_displ_box, 9, 400, 2)

; housekeeping functions for the main window

$ok_button = GUICtrlCreateButton("RUN", 100, 200, 70, 40)
GUICtrlSetOnEvent($ok_button, "Execute_Main") ; when button is pressed, proceed
$stop_button = GUICtrlCreateButton("CANCEL", 200, 200, 70, 40)
GUICtrlSetOnEvent($stop_button, "My_Final_Exit") ; ends the script

$exit_label = GUICtrlCreateLabel("Once RUN activated, Ctrl-Alt-P to Pause upon completion of current batch of exposures, Ctrl-Alt-X to Exit.", 10, 350, 380, 110)
GUICtrlSetFont($exit_label, 9, 800, 2)

GUISetOnEvent($GUI_EVENT_CLOSE, "My_Final_Exit") ; pressing X ends the script
GUISetOnEvent($GUI_EVENT_MINIMIZE, "Minimize")
GUISetOnEvent($GUI_EVENT_RESTORE, "Restore")

GUISetState(@SW_SHOW)
Wait_For_Input()

;-------------------------------------------------------------------------------------------------------------
;
;  End of main program block
;
;-------------------------------------------------------------------------------------------------------------


;******************************************************************
; this function is a loop that keeps the script going while data is being input, as well as enables restarts after a pause
;******************************************************************

Func Wait_For_Input()
	
	While 1
		
		If $script_has_started Then
			If $paused = 0 Then
				GUICtrlSetData($status_displ_box, "Error - script should not be here!")
			Else
				GUICtrlSetData($status_displ_box, "Paused. Press RUN")
			EndIf
			
		Else
			GUICtrlSetData($status_displ_box, "Press RUN to start...")
		EndIf

		Sleep(500)		; this delay tries to avoid excessive processor churning
	WEnd
EndFunc   ;==>Wait_For_Input

;********************************************************************
; enables the script to Pause
;********************************************************************

Func Pause()
	$paused = 1  ; set the Paused flag
	
	GUICtrlSetData($status_displ_box, "Will pause. Pls wait.")
	Return
EndFunc   ;==>Pause



;************************************************************************************
;
;  this contains the main loop of the program
;
;***********************************************************************************

Func Execute_Main()
	
	
	; reset the Pause status
	$paused = 0

	
	; update status
	GUICtrlSetData($status_displ_box, "Initialising")
	
	; give the user a chance to start IRIS and Artemis Capture
	
	While Not WinExists("LX200") ;		confirm that IRIS telescope panel is open
		If MsgBox(0x40015, "Error", "Pls start IRIS, turn on the Telescope panel, then press RETRY.  CANCEL to return.") = 2 Then Return
	WEnd
	
	While Not WinExists("Artemis") ;		confirm that Artemis Capture is open
		If MsgBox(0x40015, "Error", "Pls start Artemis Capture, set exposure parameters, then press RETRY.  CANCEL to return.") = 2 Then Return
	WEnd
		
	; ****************
	; the script doesn't get past this point if IRIS and Artemis Capture are not running
	; ****************
	
	; the Run and Cancel buttons stop working here
	
	GUICtrlSetState($stop_button, $GUI_DISABLE)
	GUICtrlSetState($ok_button, $GUI_DISABLE)
	
	; assign the parameters
	

	$no_exp = GUICtrlRead($no_exp_box)
	$exp_time = GUICtrlRead($exp_time_box)
	$ccd_bin = GUICtrlRead($ccd_bin_box)
	$path = GUICtrlRead($path_box)
	$targetlist_filename = GUICtrlRead($targetlist_filename_box)
	$telescope_settle_time = GUICtrlRead($telescope_settle_time_box)
	
	If StringRight($path, 1) <> "\" Then $path = $path & "\"		; make sure the path parameter has the correct form 

	;*************************************************************
	; write the parameters to file so they can be re-used
	;*************************************************************
	
	$params_file = FileOpen(@ScriptDir & "\" & "param_targetpoint.txt", 2)
	FileWriteLine($params_file, $no_exp)
	FileWriteLine($params_file, $exp_time)
	FileWriteLine($params_file, $ccd_bin)
	FileWriteLine($params_file, $path)
	FileWriteLine($params_file, $targetlist_filename)
	FileWriteLine($params_file, $telescope_settle_time)
	
	FileClose($params_file)
	
	;*************************************************************************
	; Open the target list for reading, if it's not there, exit
	;*************************************************************************
	If FileExists($targetlist_filename) Then
		$targets_total = _FileCountLines($targetlist_filename)
		$file = FileOpen($targetlist_filename, 0)
		GUICtrlSetData($targets_total_number, $targets_total)
	Else
		MsgBox(0, "Error", "The file " & $targetlist_filename & "not found.")
		Exit
	EndIf

		
	;************************************************************************************
	
	While 1; the main, infinite loop
		$return_code = 1; first set the return code to non-error
		
		$return_code = Point_and_Acquire()
		If $return_code = 0 Or $paused = 1 Then ExitLoop
	WEnd
	;************************************************************************************
	
	; the Run and Cancel buttons start working again
	
	GUICtrlSetState($stop_button, $GUI_ENABLE)
	
	If $paused = 1 Then GUICtrlSetState($ok_button, $GUI_ENABLE)		; do not re-enable the RUN button if all targets have been acquired
	
EndFunc   ;==>Execute_Main

;***********************************************
; window housekeeping functions
;***********************************************

Func Minimize()
	GUISetState(@SW_MINIMIZE)
EndFunc   ;==>Minimize

Func Restore()
	GUISetState(@SW_RESTORE)
EndFunc   ;==>Restore

Func Close_Window()
	GUIDelete()
EndFunc   ;==>Close_Window


;******************************************************************************
; Main function to do the following in order;
; - read the target list and point the telescope
; - set the image file name to be the target name and reset the file index
; - acquire the set no. of exposures for one particular target,
; - return control to the infinite loop withing Execute_Main to assess if the script
; needs to pause
;*******************************************************************************

Func Point_and_Acquire()
	
	
	; read the target name and coordinates from file
	
	GUICtrlSetData($status_displ_box, "Reading target coords...")		; update status
	Dim $target[3]
	$target = Read_Target()
	
	If $target[0] = "Done" Then			; if end of target list is reached the function exits
		GUICtrlSetData($status_displ_box, "All targets done.  Press CANCEL to exit.")
		Sleep(3000)
		Return 0
	EndIf
	
	$target_name = $target[0]
	$RA = $target[1]
	$Dec = $target[2]
	
	; records the current and previous RA in minutes
	Dim $RA_line[3]		; receives the parsed RA info, [1] is HH and [2] is MM.mm
	$RA_line = StringSplit($RA, " ")
	If $script_has_started = 1 Then
		$prev_RA_in_mins = $current_RA_in_mins
	EndIf
		
	$current_RA_in_mins = $RA_line[1]*60 + $RA_line[2]
	
	If $script_has_started = 0 Then		; ensures that if this is the first pass through, current RA = prev RA so that the Milky Way wait time is not triggered
		$prev_RA_in_mins = $current_RA_in_mins
	EndIf
	
	$script_has_started = 1		; signifies that the script has run at least once to this point.

	
	; point the telescope
	GUICtrlSetData($current_target_box, $target_name)
	GUICtrlSetData($status_displ_box, "Pointing the telescope...")		; update status
	
	WinActivate("LX200")			; makes this window appear

	Enter_RA($RA)
	Enter_Dec($Dec)
	Press_Goto()
	Sleep(1000)
	
	; enter the file path and name to be used for saving images, and reset the file index to 1
	
	WinActivate("Artemis")

	ControlSetText("Exposure", "", $Art_exp_time_ctrl, $exp_time)
	ControlCommand("Exposure", "", $Art_binx_ctrl, "SelectString", $ccd_bin)
	ControlCommand("Exposure", "", $Art_biny_ctrl, "SelectString", $ccd_bin)
	ControlCommand("Exposure", "", $Art_autosave_ctrl, "Check", "")		; make sure autosave is on
	
	ControlSetText("Exposure", "", $Art_filename_ctrl, $path & $target_name & "x")
	ControlSetText("Exposure", "", $Art_fileindex_ctrl, "1")
	GUICtrlSetData($current_target_box, $target_name)
	Sleep(1000)
	
	WinActivate("TargetPoint")
	
	; wait for the telescope to settle, or wait an additional time for target to get higher if crossing over the Milky Way
	
	If $current_RA_in_mins - $prev_RA_in_mins > $Milky_Way_wait_time Then
		$wait_time = Int($Milky_Way_wait_time)
		For $i = 1 To $wait_time
			For $j = 1 To 60
				GUICtrlSetData($status_displ_box, "Time to start of exp " & String($wait_time - $i) & " mins " & String(61 - $j) & " secs")
				Sleep(1000)
			Next
		Next
	Else
		For $j = 1 To $telescope_settle_time
			GUICtrlSetData($status_displ_box, "Time to start of exp " & String($telescope_settle_time - $j +1) & " secs")
			Sleep(1000)
		Next
	EndIf
	
	


	; update status
	GUICtrlSetData($status_displ_box, "Doing Exposures")
		
	; start the exposures
		
		
	; after pressing the Start button, we need to start the exposures 
	; then wait for the set to finish.  This is done by
	; counting the no. of times the status bar alternates between 'Downloading' and otherwise
		
	For $j = 1 To $no_exp
		
		Position_ArtCap()
		MouseClick("left", $Art_exp_x, $Art_exp_y)
		
		Do
			Sleep(200) ; the script checks to see if an exposure is done
				
		Until StatusbarGetText("Artemis", "", 5) = "Downloading"; indicates the exposure is done
			
		Do
			Sleep(200)
			
		Until StatusbarGetText("Artemis", "", 5) <> "Downloading"
		
		WinActivate("TargetPoint")		; make the GUI window visible for a while
		Sleep(2000)		;  *************CHECK TIMING  *******************
	Next
	
	;******* this bit of code updates the total no. of targets done  *******
	
	$targets_done = $targets_done + 1
	GUICtrlSetData($targets_done_box, $targets_done)
	
	;**************************************************************************
	
	GUICtrlSetData($status_displ_box, $target_name & " done")
		
	; if there was a press of Ctrl Alt P earlier, exit at this point
	If $paused = 1 Then Return 0  		
	Return 1; normal exit
	
EndFunc   ;==>Point_and_Acquire


;************************************************************************
; Function to get Artemis windows into the right position
;************************************************************************
Func Position_ArtCap()
	WinActivate("Exposure")
	WinMove("Exposure", "", 76, 339)
	;WinWaitActive("Artemis")
	WinSetState("Artemis", "", @SW_RESTORE)
	WinMove("Artemis", "", $art_winposx, $art_winposy, 718, 650) ;places main Artemis window

EndFunc   ;==>Position_ArtCap

;************************************************************************
; Functions to do various actions in the IRIS telescope control box
;************************************************************************
Func Enter_RA($RA)
	ControlSetText("LX200", "", $iris_ra_ctrl, $RA)
EndFunc   ;==>Enter_RA

Func Enter_Dec($Dec)
	ControlSetText("LX200", "", $iris_dec_ctrl, $Dec)
EndFunc   ;==>Enter_Dec

Func Press_Goto()
	ControlClick("LX200", "", $iris_goto_ctrl)
EndFunc   ;==>Press_Guide


;**************************************************************************
; Reads the next target from the target list and converts to strings
; in the form HH MM.mm and DD MM.mm
; the first value in the return array is set to "Done" if end of file is reached
;**************************************************************************

Func Read_Target()
	Dim $target_data[3]	;	this is the return array, first value is target name (must not contain a space), then RA ("HH MM.mm") and Dec ("DD MM.mm")
	Dim $input_line[6] ; this receives the parsed input line
	
	$cur_target_line = FileReadLine($file) ; the entire line containing the galaxy name (no spaces) and coordinates in the format HH MM.mm and DD MM.mm
	
	$input_line = StringSplit($cur_target_line, " ")
	
	$target_data[0] = $input_line[1]
	$target_data[1] = $input_line[2] & " " & $input_line[3]
	$target_data[2] = $input_line[4] & " " & $input_line[5]
	
	If @error = -1 Then
		FileClose($file)
		$target_data[0] = "Done"
	EndIf
	
	Return $target_data
EndFunc		;==> Read_Target


;****************************************************************************
; Browse for the file save path and update the path input field when done
;***************************************************************************

Func Browse_path()
	$newpath = FileSelectFolder("Select save path", "D:\", 1, $path)
	If $newpath <> "" Then $path = $newpath
	GUICtrlSetData($path_box, $path)
EndFunc


;****************************************************************************
; Browse for the Target List file and update the file input field when done
;***************************************************************************

Func Select_file()
	$targetlist_filename = FileOpenDialog("Select target list...", $targetlist_filename, "Text files (*.txt)" , 1 + 2 )

	If @error Then
		MsgBox(4096,"","No File(s) chosen")
	Else
		GUICtrlSetData($targetlist_filename_box, $targetlist_filename)
		$targets_total = _FileCountLines($targetlist_filename)
		GUICtrlSetData($targets_total_number, $targets_total)
	EndIf


EndFunc

;***********************************************
; exit
;************************************************
 
Func My_Final_Exit()

	Exit; this completely terminates the script and closes the interface window
EndFunc   ;==>My_Final_Exit

;**************************************************************************************************
;  END of the script.
;**************************************************************************************************